------------------------------------------------------------------- 
--                        ASCII HEX TABLE
--  Hex						Low Hex Digit
-- Value  0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
------\----------------------------------------------------------------
--H  2 |  SP  !   "   #   $   %   &   '   (   )   *   +   ,   -   .   /
--i  3 |  0   1   2   3   4   5   6   7   8   9   :   ;   <   =   >   ?
--g  4 |  @   A   B   C   D   E   F   G   H   I   J   K   L   M   N   O
--h  5 |  P   Q   R   S   T   U   V   W   X   Y   Z   [   \   ]   ^   _
--   6 |  `   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o
--   7 |  p   q   r   s   t   u   v   w   x   y   z   {   |   }   ~ DEL
-----------------------------------------------------------------------

LIBRARY IEEE;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

entity lcd_control is
	port
	(
		trigger, reset, clk		: in	std_logic;
		ascii				: in	std_logic_vector(7 downto 0);
		lcd_rs, lcd_rw, lcd_e	: out	std_logic;
		lcd_data					: inout std_logic_vector(7 downto 0);
		led : out std_logic
	);
end lcd_control;

architecture behv of lcd_control is

type state_type is (RESET1, RESET2, RESET3, 
										DROP_E, HOLD,
										FUNC_SET, DISP_ON, DISP_CLEAR, MODE_SET,
										WAIT_CHAR, CHK_BUSY);
signal state, old_state, next_command : state_type;
signal lcd_rw_int : std_logic;
signal char_cnt : std_logic_vector(4 downto 0);
signal wait_cnt : std_logic_vector(15 downto 0);
signal lcd_status : std_logic_vector (7 downto 0);
signal test, tock : std_logic;
	
begin
 	process(clk, reset)									-- Das ist die State Machine frs LCD 
	begin
  if reset = '0' then
    led <= '1';
	  state <= RESET1;
		next_command <= RESET2;							
		wait_cnt <= X"0000";
		lcd_data <= X"38";
		lcd_e <= '1';
		lcd_rs <= '0';
		lcd_rw <= '0';											-- kein Kommando an LCD
	elsif rising_edge(clk) then
	  case state is
		  when RESET1 =>									-- erstes Kommando an LCD, mind. 16ms nach Einschalten
		   	lcd_e <= '1';									 
				lcd_rs <= '0';
				lcd_rw_int <= '0';
				lcd_data <= X"38";
				if wait_cnt < X"1f40" then		-- clk period 2usec, warte 16 msec
				  wait_cnt <= wait_cnt +1;
				  state <= RESET1;
				  led <= '0';
				else
				  state <= DROP_E;						-- Kommando an LCD
					next_command <= RESET2;
					wait_cnt <= X"0000";				-- wait counter reset
				end if;
			when RESET2 =>
			  lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"38";
				if wait_cnt < X"09C4" then		-- clk period 2usec, warte 5 msec
				  wait_cnt <= wait_cnt +1;
				  state <= RESET2;
				else
				  state <= DROP_E;						-- Kommando an LCD
					next_command <= RESET3;
					wait_cnt <= X"0000";				-- wait counter reset
				end if;
			when RESET3 =>
				lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"38";
				if wait_cnt < X"003C" then		-- clk period 2usec, warte 120 usec
				  wait_cnt <= wait_cnt +1;
				  state <= RESET3;
				else
				  state <= DROP_E;						-- Kommando an LCD
					next_command <= FUNC_SET;
					wait_cnt <= X"0000";				-- wait counter reset
				end if;
			when FUNC_SET =>
				--old_state <= FUNC_SET;				-- State speichern
				--state <= CHK_BUSY;						-- warte bis LCD frei
				lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"38";
				if wait_cnt < X"001E" then		-- clk period 2usec, warte 60 usec
				  wait_cnt <= wait_cnt +1;
				  state <= FUNC_SET;
				else
				  state <= DROP_E;						-- Kommando an LCD
					next_command <= DISP_ON;
					wait_cnt <= X"0000";				-- wait counter reset
				end if;
			when DISP_ON =>
				--old_state <= FUNC_SET;				-- State speichern
				--state <= CHK_BUSY;						-- warte bis LCD frei
					lcd_e <= '1';
				lcd_rs <= '0';
				lcd_rw_int <= '0';
				if wait_cnt < X"001E" then		-- clk period 2usec, warte 60 usec
				  wait_cnt <= wait_cnt +1;
				  state <= DISP_ON;
				else
				  state <= DROP_E;						-- Kommando an LCD
					next_command <= DISP_CLEAR;
					wait_cnt <= X"0000";				-- wait counter reset
				end if;
				next_command <= DISP_CLEAR;
			when DISP_CLEAR =>
				--old_state <= FUNC_SET;				-- State speichern
				--state <= CHK_BUSY;						-- warte bis LCD frei
				lcd_e <= '1';
				lcd_rs <= '0';
				lcd_rw_int <= '0';
				if wait_cnt < X"001E" then		-- clk period 2usec, warte 60 usec
				  wait_cnt <= wait_cnt +1;
				  state <= DISP_CLEAR;
				else
				  state <= DROP_E;						-- Kommando an LCD
					next_command <= MODE_SET;
					wait_cnt <= X"0000";				-- wait counter reset
				end if;
				next_command <= MODE_SET;  
			when MODE_SET =>
				--old_state <= FUNC_SET;				-- State speichern
				--state <= CHK_BUSY;						-- warte bis LCD frei
				lcd_e <= '1';
				lcd_rs <= '0';
				lcd_rw_int <= '0';
				lcd_data <= X"06";
				state <= DROP_E;
				next_command <= WAIT_CHAR;
			when WAIT_CHAR =>								-- hier ist erstmal ENDE
				 state <= WAIT_CHAR;
			when DROP_E =>
			  lcd_e <= '0';
			  state <= HOLD;
			when HOLD =>
			  state <= next_command;
			when CHK_BUSY =>							-- prfe ob LCD busy
			  lcd_e <= '1';
				lcd_rs <= '0';
				lcd_rw <= '1';							-- lesen Statusbit vom LCD
				next_command <= CHK_BUSY;		-- Kommando an LCD
				state <= DROP_E;
				lcd_status <= lcd_data;			-- auslesen Busybit
				if lcd_status(7) = '1' then -- LCD busy
				  state <= CHK_BUSY;
				else												-- LCD frei, weiter geht's
				  lcd_rw <= '0';
				  state <= old_state;
				end if;					
			 end case;
	end if;
	end process;
	

end behv;
