------------------------------------------------------------------- 
--                        ASCII HEX TABLE
--  Hex						Low Hex Digit
-- Value  0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
------\----------------------------------------------------------------
--H  2 |  SP  !   "   #   $   %   &   '   (   )   *   +   ,   -   .   /
--i  3 |  0   1   2   3   4   5   6   7   8   9   :   ;   <   =   >   ?
--g  4 |  @   A   B   C   D   E   F   G   H   I   J   K   L   M   N   O
--h  5 |  P   Q   R   S   T   U   V   W   X   Y   Z   [   \   ]   ^   _
--   6 |  `   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o
--   7 |  p   q   r   s   t   u   v   w   x   y   z   {   |   }   ~ DEL
-----------------------------------------------------------------------

LIBRARY IEEE;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

entity lcd_control is
	port
	(
		trigger, reset, clk		: in	std_logic;
		ascii				: in	std_logic_vector(7 downto 0);
		lcd_rs, lcd_rw, lcd_e	: out	std_logic;
		lcd_data					: inout std_logic_vector(7 downto 0)  
		);
end lcd_control;

architecture behv of lcd_control is

type state_type is (RESET1, RESET2, RESET3, 
										DROP_E, HOLD,
										FUNC_SET, DISP_ON, DISP_CLEAR, MODE_SET,
										WAIT_CHAR, DISP_CLEAR_WAIT,
										WAIT_TRIGGER_RESET,
										CHK_CNT, LINE2, HOME);
signal state, next_command : state_type;
signal char_cnt : std_logic_vector(5 downto 0);
signal wait_cnt : std_logic_vector(7 downto 0);
	
begin

 	process(clk, reset)									-- Das ist die State Machine frs LCD
 
	begin
  if reset = '0' then
    state <= RESET1;
		next_command <= RESET2;							
		wait_cnt <= X"00";
		lcd_data <= X"38";
		lcd_e <= '1';
		lcd_rs <= '0';
		lcd_rw <= '0';										-- LCD wird nur zunchst nur beschrieben
		char_cnt <= "000000";
		
	elsif rising_edge(clk) then
	
	  case state is
		  when RESET1 =>									-- erstes Kommando an LCD, mind. 16ms nach Einschalten
		   	lcd_e <= '1';									 
				lcd_rs <= '0';
				lcd_data <= X"38";
				if wait_cnt < X"10" then			-- clk period 1msec, warte 16 msec
				  wait_cnt <= wait_cnt +1;
				  state <= RESET1;
				 else
				  state <= DROP_E;						-- Kommando an LCD
					next_command <= RESET2;
					wait_cnt <= X"00";					-- wait counter reset
				end if;
				
			when RESET2 =>
			  lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"38";
				if wait_cnt < X"05" then			-- clk period 1msec, warte 5 msec
				  wait_cnt <= wait_cnt +1;
				  state <= RESET2;
				else
				  state <= DROP_E;						-- Kommando an LCD
					next_command <= RESET3;
					wait_cnt <= X"00";					-- wait counter reset
				end if;
				
			when RESET3 =>
				lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"38";
				state <= DROP_E;							
				next_command <= FUNC_SET;
				
			when FUNC_SET =>
				lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"38";
			  state <= DROP_E;						
				next_command <= DISP_ON;
				
			when DISP_ON =>
				lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"0F";
			  state <= DROP_E;						
				next_command <= DISP_CLEAR;
				
			when DISP_CLEAR =>
				lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"01";
				wait_cnt <= X"00";
				state <= DROP_E;						
				next_command <= DISP_CLEAR_WAIT;
				
			when DISP_CLEAR_WAIT =>
			 	if wait_cnt < X"03" then			-- clk period 1msec, warte 3 msec
				  wait_cnt <= wait_cnt +1;
				  state <= DISP_CLEAR_WAIT;
				else
				  state <= MODE_SET;						-- Kommando an LCD
					wait_cnt <= X"00";					
				end if;
				
			when MODE_SET =>
				lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"06";
				state <= DROP_E;
				next_command <= WAIT_CHAR;
				
---------------- Initialisierung beendet, Arbeitsloop -------------------------------------------

--   WAIT_CHAR -> DROP_E -> HOLD -> WAIT_TRIGGER_RESET -> CHK_CNT -> LINE2 -> DROP_E -> HOLD -> |
--																																|															|
--        ^                                                       -> HOME  -> DROP_E -> HOLD -> |
--        |                                                       |															|
--        |                                                       v                             v
--        |----------------------------------------<<-------------------------------------------|

-------------------------------------------------------------------------------------------------
				
			when WAIT_CHAR =>
				if trigger = '0' then
			    lcd_e <= '1';
					lcd_rs <= '1';
					lcd_data <= ascii;
					char_cnt <= char_cnt +1;
					state <= DROP_E;
					next_command <= WAIT_TRIGGER_RESET;
				else
				  next_command <= WAIT_CHAR;
				end if;
				
			when WAIT_TRIGGER_RESET =>
			  if trigger = '0' then
			   	state <= WAIT_TRIGGER_RESET;
			  else
					state <= CHK_CNT;
				end if;
				
			when CHK_CNT =>
			  if char_cnt = X"10" then
			    state <= LINE2;
			  elsif char_cnt = X"20" then
				  state <= HOME;
				else
				  state <= WAIT_CHAR;
				end if;
				
				
			when LINE2 =>
			  lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"C0";
				state <= DROP_E;
				next_command <= WAIT_CHAR;
				
			when HOME =>
		   	lcd_e <= '1';
				lcd_rs <= '0';
				lcd_data <= X"80";
				char_cnt <= "000000";
				state <= DROP_E;
				next_command <= WAIT_CHAR;
							
			when DROP_E =>			
			  lcd_e <= '0';
			  state <= HOLD;
			
			when HOLD =>
			  state <= next_command;
			
		end case;
	end if;
	end process;
	

end behv;
